@@ -5,6 +5,7 @@ Custom element classes related to text, such as paragraph (CT_P) and runs
 (CT_R).
 """
 
+from docx.enum.text import WD_UNDERLINE
 from docx.oxml.parts.numbering import CT_NumPr
 from docx.oxml.shared import (
     CT_String, nsdecls, OxmlBaseElement, OxmlElement, oxml_fromstring, qn
@@ -294,6 +295,22 @@ class CT_R(OxmlBaseElement):
         """
         return self.findall(qn('w:t'))
 
+    @property
+    def underline(self):
+        """
+        String contained in w:val attribute of <w:u> grandchild, or |None| if
+        that element is not present.
+        """
+        rPr = self.rPr
+        if rPr is None:
+            return None
+        return rPr.underline
+
+    @underline.setter
+    def underline(self, value):
+        rPr = self.get_or_add_rPr()
+        rPr.underline = value
+
     def _add_rPr(self):
         """
         Return a newly added rPr child element. Assumes one is not present.
@@ -653,6 +670,11 @@ class CT_RPr(OxmlBaseElement):
         for strike in strike_lst:
             self.remove(strike)
 
+    def remove_u(self):
+        u_lst = self.findall(qn('w:u'))
+        for u in u_lst:
+            self.remove(u)
+
     def remove_vanish(self):
         vanish_lst = self.findall(qn('w:vanish'))
         for vanish in vanish_lst:
@@ -737,6 +759,31 @@ class CT_RPr(OxmlBaseElement):
         else:
             self.rStyle.val = style
 
+    @property
+    def u(self):
+        """
+        First ``<w:u>`` child element or |None| if none are present.
+        """
+        return self.find(qn('w:u'))
+
+    @property
+    def underline(self):
+        """
+        Underline type specified in <w:u> child, or None if that element is
+        not present.
+        """
+        u = self.u
+        if u is None:
+            return None
+        return u.val
+
+    @underline.setter
+    def underline(self, value):
+        self.remove_u()
+        if value is not None:
+            u = self._add_u()
+            u.val = value
+
     @property
     def vanish(self):
         """
@@ -756,6 +803,14 @@ class CT_RPr(OxmlBaseElement):
         self.insert(0, rStyle)
         return rStyle
 
+    def _add_u(self):
+        """
+        Return a newly added <w:u/> child element.
+        """
+        u = OxmlElement('w:u')
+        self.insert(0, u)
+        return u
+
 
 class CT_Text(OxmlBaseElement):
     """
@@ -769,3 +824,62 @@ class CT_Text(OxmlBaseElement):
         t = OxmlElement('w:t')
         t.text = text
         return t
+
+
+class CT_Underline(OxmlBaseElement):
+    """
+    ``<w:u>`` element, specifying the underlining style for a run.
+    """
+    @property
+    def val(self):
+        """
+        The underline type corresponding to the ``w:val`` attribute value.
+        """
+        underline_type_map = {
+            None:              None,
+            'none':            False,
+            'single':          True,
+            'words':           WD_UNDERLINE.WORDS,
+            'double':          WD_UNDERLINE.DOUBLE,
+            'dotted':          WD_UNDERLINE.DOTTED,
+            'thick':           WD_UNDERLINE.THICK,
+            'dash':            WD_UNDERLINE.DASH,
+            'dotDash':         WD_UNDERLINE.DOT_DASH,
+            'dotDotDash':      WD_UNDERLINE.DOT_DOT_DASH,
+            'wave':            WD_UNDERLINE.WAVY,
+            'dottedHeavy':     WD_UNDERLINE.DOTTED_HEAVY,
+            'dashedHeavy':     WD_UNDERLINE.DASH_HEAVY,
+            'dashDotHeavy':    WD_UNDERLINE.DOT_DASH_HEAVY,
+            'dashDotDotHeavy': WD_UNDERLINE.DOT_DOT_DASH_HEAVY,
+            'wavyHeavy':       WD_UNDERLINE.WAVY_HEAVY,
+            'dashLong':        WD_UNDERLINE.DASH_LONG,
+            'wavyDouble':      WD_UNDERLINE.WAVY_DOUBLE,
+            'dashLongHeavy':   WD_UNDERLINE.DASH_LONG_HEAVY,
+        }
+        val = self.get(qn('w:val'))
+        return underline_type_map[val]
+
+    @val.setter
+    def val(self, value):
+        underline_vals = {
+            True:                            'single',
+            False:                           'none',
+            WD_UNDERLINE.WORDS:              'words',
+            WD_UNDERLINE.DOUBLE:             'double',
+            WD_UNDERLINE.DOTTED:             'dotted',
+            WD_UNDERLINE.THICK:              'thick',
+            WD_UNDERLINE.DASH:               'dash',
+            WD_UNDERLINE.DOT_DASH:           'dotDash',
+            WD_UNDERLINE.DOT_DOT_DASH:       'dotDotDash',
+            WD_UNDERLINE.WAVY:               'wave',
+            WD_UNDERLINE.DOTTED_HEAVY:       'dottedHeavy',
+            WD_UNDERLINE.DASH_HEAVY:         'dashedHeavy',
+            WD_UNDERLINE.DOT_DASH_HEAVY:     'dashDotHeavy',
+            WD_UNDERLINE.DOT_DOT_DASH_HEAVY: 'dashDotDotHeavy',
+            WD_UNDERLINE.WAVY_HEAVY:         'wavyHeavy',
+            WD_UNDERLINE.DASH_LONG:          'dashLong',
+            WD_UNDERLINE.WAVY_DOUBLE:        'wavyDouble',
+            WD_UNDERLINE.DASH_LONG_HEAVY:    'dashLongHeavy',
+        }
+        val = underline_vals[value]
+        self.set(qn('w:val'), val)
