@@ -9,7 +9,7 @@ from __future__ import absolute_import, print_function, unicode_literals
 from behave import given, then, when
 
 from docx import Document
-from docx.enum.text import WD_BREAK
+from docx.enum.text import WD_BREAK, WD_UNDERLINE
 from docx.oxml.shared import qn
 
 from .helpers import test_docx, test_text
@@ -30,6 +30,15 @@ def given_a_run_having_bool_prop_set_on(context, bool_prop_name):
     context.run = run
 
 
+@given('a run having {underline_type} underline')
+def given_a_run_having_underline_type(context, underline_type):
+    run_idx = {
+        'inherited': 0, 'no': 1, 'single': 2, 'double': 3
+    }[underline_type]
+    document = Document(test_docx('run-enumerated-props'))
+    context.run = document.paragraphs[0].runs[run_idx]
+
+
 @given('a run having style {char_style}')
 def given_a_run_having_style_char_style(context, char_style):
     run_idx = {
@@ -84,6 +93,16 @@ def when_I_set_the_character_style_of_the_run(context, char_style):
     context.run.style = style_value
 
 
+@when('I set the run underline to {underline_value}')
+def when_I_set_the_run_underline_to_value(context, underline_value):
+    new_value = {
+        'True': True, 'False': False, 'None': None,
+        'WD_UNDERLINE.SINGLE': WD_UNDERLINE.SINGLE,
+        'WD_UNDERLINE.DOUBLE': WD_UNDERLINE.DOUBLE,
+    }[underline_value]
+    context.run.underline = new_value
+
+
 # then =====================================================
 
 @then('it is a column break')
@@ -137,6 +156,15 @@ def then_the_run_contains_the_text_I_specified(context):
     assert context.run.text == test_text
 
 
+@then('the run underline property value is {underline_value}')
+def then_the_run_underline_property_value_is(context, underline_value):
+    expected_value = {
+        'None': None, 'False': False, 'True': True,
+        'WD_UNDERLINE.DOUBLE': WD_UNDERLINE.DOUBLE
+    }[underline_value]
+    assert context.run.underline == expected_value
+
+
 @then('the style of the run is {char_style}')
 def then_the_style_of_the_run_is_char_style(context, char_style):
     expected_value = {
